@@ -4,8 +4,8 @@
 
 ;; Authors: Ashish Barnawal, Diego Riviero, Gabriel Hondet, Rodolphe Lepigre
 ;; Maintainer: Deducteam <dedukti-dev@inria.fr>
-;; Package-Version: 20250716.1143
-;; Package-Revision: f73c64dbeebf
+;; Package-Version: 20260203.1712
+;; Package-Revision: 4fd4704bf773
 ;; SPDX-License-Identifier: CECILL-2.1
 ;; Homepage: https://github.com/Deducteam/lambdapi
 ;; Keywords: languages
@@ -39,6 +39,8 @@
         (modify-syntax-entry ?\; ". 23" syn-table)
         syn-table))
 
+(setq short-diags-enabled nil)
+
 ;; Keywords (legacy syntax)
 (defconst lambdapi-legacy-font-lock-keywords
   (list
@@ -122,6 +124,7 @@
   (define-key lambdapi-mode-map (kbd "C-c C-f") #'lp-jump-proof-forward)
   (define-key lambdapi-mode-map (kbd "C-c C-b") #'lp-jump-proof-backward)
   (define-key lambdapi-mode-map (kbd "C-c C-r") #'lambdapi-eglot-reconnect)
+  (define-key lambdapi-mode-map (kbd "C-c C-v") #'toggle-short-diagnostics)
   (define-key lambdapi-mode-map (kbd "C-c C-k") #'eglot-shutdown)
   ;; define toolbar
   (define-key lambdapi-mode-map [tool-bar lp-toggle-electric-terminator]
@@ -153,6 +156,49 @@
   "LambdaPi is a proof assistant based on the λΠ-calculus modulo rewriting"
   :group 'languages)
 
+;; preprocesses (typically, shortens positions of) diagnostics before passing them to Eglot for display
+(defun pre-process-diagnostics  (orig-fun server method uri aPath_string e diagnostics)
+(dotimes (i (length diagnostics))
+  (let* (
+        (aDiagnostic (aref diagnostics i))
+        (range (plist-get aDiagnostic :range))
+        (end (plist-get range :end))
+        (end_character (plist-get end :character))
+        (start (plist-get range :start))
+        (start_character (plist-get start :character))
+        (start_line (plist-get start :line)))
+  (setq end (plist-put end :character (min end_character (+ 3 start_character))))
+  (setq end (plist-put end :line start_line)) 
+  )  
+)
+  (apply orig-fun (list server method uri aPath_string e diagnostics))
+)
+
+;;;###autoload
+(defun activate-short-diagnostics ()
+  (interactive)
+  (advice-add 'eglot-handle-notification :around #'pre-process-diagnostics)
+  (setq short-diags-enabled t)
+  (message "Lambdapi: diagnostic shortning enabled.")
+)
+
+;;;###autoload
+(defun deactivate-short-diagnostics ()
+  (interactive)
+  (advice-remove 'eglot-handle-notification #'pre-process-diagnostics)
+  (setq short-diags-enabled nil)
+  (message "Lambdapi: diagnostic shortning disabled.")
+)
+
+;;;###autoload
+(defun toggle-short-diagnostics ()
+  (interactive)
+  (if short-diags-enabled
+      (deactivate-short-diagnostics)
+    (activate-short-diagnostics)
+  )
+)
+
 ;; Main function creating the mode (lambdapi)
 ;;;###autoload
 (define-derived-mode lambdapi-mode prog-mode "LambdaPi"
@@ -210,5 +256,7 @@
 ;;;###autoload
 (add-to-list 'auto-mode-alist '("\\.dk\\'" . lambdapi-legacy-mode))
 
+(activate-short-diagnostics)
+
 (provide 'lambdapi-mode)
 ;;; lambdapi-mode.el ends here
